<?php

namespace Drupal\drupal_marketplace\Plugin\rest\resource;

use Drupal\rest\ModifiedResourceResponse;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Drupal\rest\ResourceResponseInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\HttpKernel\Exception\ServiceUnavailableHttpException;

/**
 * Provides a resource to get view modes by entity and bundle.
 *
 * @RestResource(
 *   id = "package_rest_resource",
 *   label = @Translation("Manage Drupal Extestions"),
 *   uri_paths = {
 *     "canonical" = "/package",
 *     "create" = "/package"
 *   },
 *   no_cache = TRUE
 * )
 */
class PackageRestResource extends ResourceBase {

  /**
   * A current user instance.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * Composer wrapper service.
   *
   * @var \Drupal\drupal_marketplace\Service\ComposerWrapper
   */
  protected $composerWrapper;

  /**
   * Projects manager.
   *
   * @var \Drupal\drupal_marketplace\Service\ProjectsManager
   */
  protected $projectsManager;

  /**
   * Module installer.
   *
   * @var \Drupal\Core\Extension\ModuleInstaller
   */
  protected $moduleInstaller;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->logger = $container->get('logger.factory')->get('drupal_marketplace');
    $instance->currentUser = $container->get('current_user');
    $instance->composerWrapper = $container->get('drupal_marketplace.composer_wrapper');
    $instance->projectsManager = $container->get('drupal_marketplace.projects_manager');
    $instance->moduleInstaller = $container->get('module_installer');
    return $instance;
  }

  /**
   * Responds to GET requests.
   *
   * @return \Drupal\rest\ResourceResponseInterface
   *   The HTTP response object.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Throws exception expected.
   */
  public function get(): ResourceResponseInterface {
    // You must to implement the logic of your REST Resource here.
    // Use current user after pass authentication to validate access.
    if (!$this->currentUser->hasPermission('access content')) {
//      throw new AccessDeniedHttpException();
    }
    $packages = $this->projectsManager->getProjectsList(\Drupal::request()->query);
    return new ResourceResponse($packages, 200);
  }

  /**
   * Execute require command.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Package name.
   *
   * @return \Drupal\rest\ResourceResponseInterface
   *   Response.
   */
  public function post(Request $request): ResourceResponseInterface {
    if (!$this->currentUser->hasPermission('administer modules')) {
      throw new AccessDeniedHttpException();
    }
    $data = json_decode($request->getContent());
    $package_name = $data->package_name;
    if (!isset($package_name)) {
      throw new BadRequestHttpException('No payload received.');
    }
    try {
      $result = $this->composerWrapper->requireCommand('drupal/' . $package_name);
      /** @var \Drupal\Core\Extension\ModuleExtensionList $moduleExtensionList */
      $moduleExtensionList = \Drupal::service('extension.list.module');
      $moduleExtensionList->reset();
      /**
       * Now it's not possible to install module. Enable it once issue resolved.
       *
       * @see https://www.drupal.org/project/drupal/issues/2899837
       **/
//      if ($moduleExtensionList->reset()->exists($package_name)) {
//        $this->moduleInstaller->install([$package_name]);
//      }
    }
    catch (\Exception $e) {
      throw new ServiceUnavailableHttpException(0, 'There was an exception: ' . $e->getMessage());
    }

    // $result['code'] is 0 if installed, 2 if not installed. 1 - ?
    if ($result['code'] === 0) {
      $message = $this->t(
        'Module :module installed successfully. Please enable it with Drush or in admin panel.',
        [':module' => $package_name]
      );
    }
    else {
      $message = $this->t(
        'Failed to install :module. Please see detailed Composer logs bellow.',
        [':module' => $package_name]
      );
    }

    $result = [
      'message' => $message,
      'composer_message' => str_replace(['<warning>', '</warning>'], ['<p>', '</p>'], $result['message']) ?? '',
      'status' => $result['code'] ?? 0,
    ];
    return new ModifiedResourceResponse($result, 200);
  }

  /**
   * Delete.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Package name.
   *
   * @return \Drupal\rest\ResourceResponseInterface
   *   Response.
   */
  public function delete(Request $request) {
    if (!$this->currentUser->hasPermission('administer modules')) {
      throw new AccessDeniedHttpException();
    }
    $package_name = $request->get('package-name');
    if (!isset($package_name)) {
      throw new BadRequestHttpException('No payload received.');
    }

    try {
      $result = $this->composerWrapper->removeCommand('drupal/' . $package_name);
    }
    catch (\Exception $e) {
      throw new ServiceUnavailableHttpException('There was an exception: ' . $e->getMessage());
    }

    $results = [
      'message' => $result['message'] ?? 'empty message',
      'status' => $result['code'] ?? 0,
    ];
    return new ModifiedResourceResponse($results, 200);

  }

}
