<?php

namespace Drupal\drupal_marketplace\Service;

use Drupal\Core\Http\InputBag;
use Drupal\drupal_marketplace\Entity\Package;
use GuzzleHttp\TransferStats;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

class ProjectsManager {

  protected $eventDispatcher;

  protected $httpClient;

  public function __construct(
    EventDispatcherInterface $event_dispatcher,
    HttpClient $http_client
  ) {
    $this->eventDispatcher = $event_dispatcher;
    $this->httpClient = $http_client;
  }


  /**
   * Get list of packages.
   *
   * @return array
   *   Packages.
   */
  public function getProjectsList(InputBag $query): array {
    $packages = [];
    $params = $query->all();
    unset($params['_format']);
    $params['type'] = 'project_module';
    $params['status'] = '1';
    // It decreases number of pages from 37K to <700. TODO Check if it's valid.
    $params['field_project_has_releases'] = TRUE;
    $response = $this->getProjects($params);

    foreach ($response['list'] as $module) {
      // TODO question - try construct a composer Package instead of custom one? too many fields :)
      $package = new Package($module);
      $project_response = $this->getProjectReleases($package->name);
      $package->setReleases($project_response['releases']);
      $this->parseMaintenanceStatus($package);
      $this->parseDevelopmentStatus($package);
      $this->parseCategories($package);
      if (count($package->images)) {
        $image = reset($package->images);
        $package->setImageUrl($this->getProjectImageUrl($image['file']['id']));
      }
      // Exclude incompatible packages.
      if ($package->isCompatible) {
        $packages[$package->name] = $package;
      }
    }
    $response = [
      'pages' => $this->extractPagesFromResponse($response),
      'packages' => $packages,
    ];
    return $response;
  }

  public function parseMaintenanceStatus(Package $package): void {
    if ($package->taxonomy_vocabulary_44) {
      $url = $package->taxonomy_vocabulary_44['uri'] . '.json';
      $response = $this->httpClient->getGuzzleClient()->request('GET', $url);
      if ($response->getStatusCode() !== 200) {
        throw new \RuntimeException("Request to $url failed, returned {$response->getStatusCode()} with reason: {$response->getReasonPhrase()}");
      }
      $response = json_decode($response->getBody()
        ->getContents(), TRUE, 512, JSON_THROW_ON_ERROR);
      $package->setMaintenanseStatus($response['name']);
      unset($package->taxonomy_vocabulary_44);
    }
  }

  public function parseDevelopmentStatus(Package $package): void {
    if ($package->taxonomy_vocabulary_46) {
      $url = $package->taxonomy_vocabulary_46['uri'] . '.json';
      $response = $this->httpClient->getGuzzleClient()->request('GET', $url);
      if ($response->getStatusCode() !== 200) {
        throw new \RuntimeException("Request to $url failed, returned {$response->getStatusCode()} with reason: {$response->getReasonPhrase()}");
      }
      $response = json_decode($response->getBody()
        ->getContents(), TRUE, 512, JSON_THROW_ON_ERROR);
      $package->setDevelopmentStatus($response['name']);
      unset($package->taxonomy_vocabulary_46);
    }
  }

  public function parseCategories(Package $package): void {
    if ($package->taxonomy_vocabulary_3) {
      $categories = [];
      foreach ($package->taxonomy_vocabulary_3 as $term) {
        $url = $term['uri'] . '.json';
        $response = $this->httpClient->getGuzzleClient()->request('GET', $url);
        if ($response->getStatusCode() !== 200) {
          throw new \RuntimeException("Request to $url failed, returned {$response->getStatusCode()} with reason: {$response->getReasonPhrase()}");
        }
        $response = json_decode($response->getBody()
          ->getContents(), TRUE, 512, JSON_THROW_ON_ERROR);
        if (isset($response['name'])) {
          $categories[] = $response['name'];
        }
      }
      $package->setCategories($categories);
      unset($package->taxonomy_vocabulary_3);
    }
  }

  public function extractPagesFromResponse(array $response) {
    $pages = [
      'current' => 0,
      'first' => 0,
      'last' => 0,
      'next' => 0,
    ];
    $pages['last'] = (int) explode('&', explode('page=', $response['last'])[1])[0];
    $pages['next'] = (int) explode('&', explode('page=', $response['next'])[1])[0];
    if (!empty(explode('&', explode('page=', $response['self'])[1])[0])) {
      $pages['current'] = (int) explode('&', explode('page=', $response['self'])[1])[0];
    }
    return $pages;
  }

  public function getProjects($query = []) {
    $client = $this->httpClient->getGuzzleClient();
    $response = $client->request('GET', "https://www.drupal.org/api-d7/node.json", [
      'on_stats' => static function (TransferStats $stats) use (&$url) {
        $url = $stats->getEffectiveUri();
      },
      'query' => $query
    ]);

    if ($response->getStatusCode() !== 200) {
      throw new \RuntimeException("Request to $url failed, returned {$response->getStatusCode()} with reason: {$response->getReasonPhrase()}");
    }
    return json_decode($response->getBody()->getContents(), true, 512, JSON_THROW_ON_ERROR);
  }

  /**
   * Requests a node from the Drupal.org API.
   *
   * @param string $project
   *   The Drupal.org project to get the releases from.
   *
   * @return array
   *   An array releases.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException Thrown if request is unsuccessful.
   */
  public function getProjectReleases(string $project) {
    if ($project === 'drupal/core') {
      $project = 'drupal';
    }
    else {
      $project = str_replace(['drupal/', 'acquia/'], '', $project);
    }
    $response = $this->requestProjectReleases($project);
    if (array_key_exists('releases', $response)) {
      // Only one release.
      if (array_key_exists('name', $response['releases']['release'])) {
        $response['releases'] = [$response['releases']['release']];
      }
      // Multiple releases.
      else {
        $response['releases'] = $response['releases']['release'];
      }
    }
    // No releases.
    else {
      $response['releases'] = [];
    }

    return $response;
  }

  /**
   * Requests a node from the Drupal.org API.
   *
   * @param string $project
   *   The Drupal.org project name.
   *
   * @return array The response object.
   *   The response object.
   *
   * @throws \GuzzleHttp\Exception\GuzzleException Thrown if request is unsuccessful.
   * @see https://www.drupal.org/drupalorg/docs/apis/rest-and-other-apis#s-releases
   * @see https://www.drupal.org/drupalorg/docs/apis/update-status-xml
   */
  protected function requestProjectReleases(string $project): array {
    $url = "https://updates.drupal.org/release-history/$project/current";
    $client = $this->httpClient->getGuzzleClient();
    $response = $client->request('GET', $url);
    if ($response->getStatusCode() !== 200) {
      throw new \RuntimeException("Request to $url failed, returned {$response->getStatusCode()} with reason: {$response->getReasonPhrase()}");
    }
    $body = $response->getBody()->getContents();
    if (strpos($body, 'No release history was found for the requested project') !== FALSE) {
      return [];
    }

    $xml = simplexml_load_string($body);
    $json = json_decode(json_encode($xml), TRUE);

    return $json;
  }

  protected function getProjectImageUrl(int $fid) {
    $url = 'https://www.drupal.org/api-d7/file/' . $fid . '.json';
    $client = $this->httpClient->getGuzzleClient();
    $response = $client->request('GET', $url);
    if ($response->getStatusCode() !== 200) {
      throw new \RuntimeException("Request to $url failed, returned {$response->getStatusCode()} with reason: {$response->getReasonPhrase()}");
    }
    $body = $response->getBody()->getContents();
    $json = json_decode($body);

    return $json->url;
  }

}
